/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "c4x4key.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define C4X4KEY_DUMMY 0

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 


// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void c4x4key_cfg_setup ( c4x4key_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

C4X4KEY_RETVAL c4x4key_init ( c4x4key_t *ctx, c4x4key_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return C4X4KEY_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, C4X4KEY_DUMMY );

    return C4X4KEY_OK;
}

void c4x4key_generic_transfer ( c4x4key_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_stop(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_start(&ctx->spi);    
}

uint16_t c4x4key_get_data ( c4x4key_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t tx_buf;
    uint8_t rx_buf[ 2 ];
    uint16_t result;

    tx_buf = C4X4KEY_DUMMY;
    
    block.tx_buffer = &tx_buf;
    block.tx_length = 0;
    block.rx_buffer = rx_buf;
    block.rx_length = 2;

    c4x4key_generic_transfer( ctx, &block );
    
    result = rx_buf[ 0 ];
    result <<= 8;
    result |= rx_buf[ 1 ];
    
    return result;
}

uint8_t c4x4key_get_btn_position ( c4x4key_t *ctx )
{
    uint16_t result;
    uint8_t position;
    
    position = 0;

    result = c4x4key_get_data ( ctx );

    result = C4X4KEY_MAX_16_BIT - result;

    while ( result )
    {
        position++;
        result >>= 1;
    }
    
    return position;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS


// ------------------------------------------------------------------------- END

